/** @file modbuscli.c MODBUS/UDP client sample for the MSXE301
*
* FC16 queries are for setting things, FC3 for fetching values
*
* This code doesn't handle timeouts.
*/

#ifdef WIN32
#include "../../Interface_Library/msxe301x_modbus_clientlib.h"
#else
#include <msxe301x_modbus_clientlib.h>
#endif

#include <modbusclient.h>
#include <commonclient.h>

#include <addidata_console.h>
#include <addidata_time.h>
#include <addidata_network.h>

#define MFMARK(__fn) printf("calling MODBUS function %s\n",__fn)

//---------------------------------------------------------------------------------
int sample_MSXE301x__AnalogInputGetChannelType(struct modbus * modbus)
{
	struct MSXE301x__AnalogInputGetChannelType_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	MFMARK("MSXE301x__AnalogInputGetChannelType");
	ret = modbus_call_MSXE301x__AnalogInputGetChannelType(modbus, &Response);

	if ( !handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE301x__AnalogInputGetChannelType") )
	{

		{
			int i;
			for(i=0;i<16;i++)
				printf("\t%X",Response.ulType[i]);
			printf("\n");
		}
		return 0;
	}
	return -1;
}
//---------------------------------------------------------------------------------
int sample_MSXE301x__AnalogInputGetChannelTypeEx(struct modbus * modbus)
{
	struct MSXE301x__AnalogInputGetChannelTypeEx_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	MFMARK("MSXE301x__AnalogInputGetChannelTypeEx");
	ret = modbus_call_MSXE301x__AnalogInputGetChannelTypeEx(modbus, &Response);

	if ( !handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE301x__AnalogInputGetChannelTypeEx") )
	{

		{
			int i;
			for(i=0;i<16;i++)
				printf("\t%X",Response.ulType[i]);
			printf("\n");
		}
		return 0;
	}
	return -1;
}

//---------------------------------------------------------------------------------
int sample_MSXE301X__TransducerInitAndStartAutoRefresh(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulChannelMask = 0xFFFF;
	uint32_t pulGainArray[16] =
		{
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1
		};
	uint32_t pulPolarityArray[16] =
		{
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0
		};
	uint32_t ulAverageMode = 0;
	uint32_t ulAverageValue = 0;
	uint32_t ulConversionTime = 1000;
	uint32_t ulConversionTimeUnit = 0; /* micro */
	uint32_t ulTriggerMask = 0;
	uint32_t ulTriggerMode = 0;
	uint32_t ulHardwareTriggerEdge = 0;
	uint32_t ulHardwareTriggerCount = 0;
	uint32_t ulByTriggerNbrOfSeqToAcquire = 0;
	uint32_t ulDataFormat = 0;
	uint32_t ulOption1 = 0;
	uint32_t ulOption2 = 0;
	uint32_t ulOption3 = 0;

	MFMARK("MSXE301x__AnalogInputInitAndStartAutoRefresh");
	ret = modbus_call_MSXE301x__AnalogInputInitAndStartAutoRefresh(modbus,
			ulChannelMask,
			pulGainArray,
			pulPolarityArray,
			ulAverageMode,
			ulAverageValue,
			ulConversionTime,
			ulConversionTimeUnit,
			ulTriggerMask,
			ulTriggerMode,
			ulHardwareTriggerEdge,
			ulHardwareTriggerCount,
			ulByTriggerNbrOfSeqToAcquire,
			ulDataFormat,
			ulOption1,
			ulOption2,
			ulOption3,
			&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE301x__AnalogInputInitAndStartAutoRefresh" );

}
//---------------------------------------------------------------------------------
int sample_MSXE301X__TransducerInitAndStartAutoRefreshEx(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulChannelMask = 0xFFFF;
	uint32_t pulGainArray[16] =
		{
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1
		};
	uint32_t pulPolarityArray[16] =
		{
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0
		};
	uint32_t ulAverageMode = 0;
	uint32_t ulAverageValue = 0;
	uint32_t ulConversionTime = 1000;
	uint32_t ulConversionTimeUnit = 0; /* micro */
	uint32_t ulTriggerMask = 0;
	uint32_t ulTriggerMode = 0;
	uint32_t ulHardwareTriggerEdge = 0;
	uint32_t ulHardwareTriggerCount = 0;
	uint32_t ulByTriggerNbrOfSeqToAcquire = 0;
	uint32_t ulDataFormat = 0;
	uint32_t ulOption1 = 0;
	uint32_t ulOption2 = 0;
	uint32_t ulOption3 = 0;

	MFMARK("MSXE301x__AnalogInputInitAndStartAutoRefreshEx");
	ret = modbus_call_MSXE301x__AnalogInputInitAndStartAutoRefreshEx(modbus,
			ulChannelMask,
			pulGainArray,
			pulPolarityArray,
			ulAverageMode,
			ulAverageValue,
			ulConversionTime,
			ulConversionTimeUnit,
			ulTriggerMask,
			ulTriggerMode,
			ulHardwareTriggerEdge,
			ulHardwareTriggerCount,
			ulByTriggerNbrOfSeqToAcquire,
			ulDataFormat,
			ulOption1,
			ulOption2,
			ulOption3,
			&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE301x__AnalogInputInitAndStartAutoRefreshEx" );

}
//---------------------------------------------------------------------------------
int sample_MSXE301x__AnalogInputGetAutoRefreshValues(struct modbus * modbus)
{
	struct MSXE301x__AnalogInputGetAutoRefreshValues_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MSXE301x__AnalogInputGetAutoRefreshValues(modbus, &Response);

	if ( !handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE301x__AnalogInputGetAutoRefreshValues") )
	{

		printf("%u\t%u\t%u",Response.ulCounterValue,Response.ulTimeStampLow,Response.ulTimeStampHigh);
		{
			int i;
			for(i=0;i<16;i++)
				printf("\t%X",Response.ulValue[i]);
			printf("\n");
		}
		return 0;
	}
	return -1;
}
//---------------------------------------------------------------------------------
int sample_MSXE301x__AnalogInputGetAutoRefreshValuesEx(struct modbus * modbus)
{
	struct MSXE301x__AnalogInputGetAutoRefreshValuesEx_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MSXE301x__AnalogInputGetAutoRefreshValuesEx(modbus, &Response);

	if ( !handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE301x__AnalogInputGetAutoRefreshValuesEx") )
	{

		printf("%u\t%u\t%u",Response.ulCounterValue,Response.ulTimeStampLow,Response.ulTimeStampHigh);
		{
			int i;
			for(i=0;i<16;i++)
				printf("\t%X",Response.ulValue[i]);
			printf("\n");
		}
		return 0;
	}
	return -1;
}

//---------------------------------------------------------------------------------
int sample_MSXE301x__AnalogInputStopAndReleaseAutoRefresh(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	MFMARK("MSXE301x__AnalogInputStopAndReleaseAutoRefresh");
	ret = modbus_call_MSXE301x__AnalogInputStopAndReleaseAutoRefresh(modbus,Dummy,&CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE301x__AnalogInputStopAndReleaseAutoRefresh");

}
//---------------------------------------------------------------------------------
int sample_MSXE301x__AnalogInputStopAndReleaseAutoRefreshEx(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	MFMARK("MSXE301x__AnalogInputStopAndReleaseAutoRefreshEx");
	ret = modbus_call_MSXE301x__AnalogInputStopAndReleaseAutoRefreshEx(modbus,Dummy,&CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE301x__AnalogInputStopAndReleaseAutoRefreshEx");

}

//---------------------------------------------------------------------------------
int sample_MSXE301X__AnalogInputInitAndStartSequence(struct modbus * modbus)
{
	//Remote function call
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulNbrOfChannel = 16;
	uint32_t pulChannelList[16] =
		{
				0,
				1,
				2,
				3,
				4,
				5,
				6,
				7,
				8,
				9,
				10,
				11,
				12,
				13,
				14,
				15
		};
	uint32_t pulGainArray[16] =
		{
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1
		};
	uint32_t pulPolarityArray[16] =
		{
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0
		};
	uint32_t ulConversionTime = 10;
	uint32_t ulConversionTimeUnit = 0;
	uint32_t ulNbrOfSequence = 0;
	uint32_t ulNbrMaxSequenceToTransfer = 10;
	uint32_t ulDelayMode = 0;
	uint32_t ulDelayTimeUnit = 0;
	uint32_t ulDelayValue = 0;
	uint32_t ulTriggerMask = 0;
	uint32_t ulTriggerMode = 0;
	uint32_t ulHardwareTriggerEdge = 1;
	uint32_t ulHardwareTriggerCount = 0;
	uint32_t ulByTriggerNbrOfSeqToAcquire = 0;
	uint32_t ulDataFormat = 0;

	MFMARK("MSXE301x__AnalogInputInitAndStartSequence");
	ret = modbus_call_MSXE301x__AnalogInputInitAndStartSequence(modbus,
			ulNbrOfChannel,
			pulChannelList,
			pulGainArray,
			pulPolarityArray,
			ulConversionTime,
			ulConversionTimeUnit,
			ulNbrOfSequence,
			ulNbrMaxSequenceToTransfer,
			ulDelayMode,
			ulDelayTimeUnit,
			ulDelayValue,
			ulTriggerMask,
			ulTriggerMode,
			ulHardwareTriggerEdge,
			ulHardwareTriggerCount,
			ulByTriggerNbrOfSeqToAcquire,
			ulDataFormat,
			0,
			&CommandStatus
	);
	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSX301x__TransducerInitAndStartSequence" );
}
//---------------------------------------------------------------------------------
int sample_MSXE301X__AnalogInputInitAndStartSequenceEx(struct modbus * modbus)
{
	//Remote function call
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulNbrOfChannel = 16;
	uint32_t pulChannelList[16] =
		{
				0,
				1,
				2,
				3,
				4,
				5,
				6,
				7,
				8,
				9,
				10,
				11,
				12,
				13,
				14,
				15
		};
	uint32_t pulGainArray[16] =
		{
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1,
				1
		};
	uint32_t pulPolarityArray[16] =
		{
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0,
				0
		};
	uint32_t ulConversionTime = 10;
	uint32_t ulConversionTimeUnit = 0;
	uint32_t ulNbrOfSequence = 0;
	uint32_t ulNbrMaxSequenceToTransfer = 10;
	uint32_t ulDelayMode = 0;
	uint32_t ulDelayTimeUnit = 0;
	uint32_t ulDelayValue = 0;
	uint32_t ulTriggerMask = 0;
	uint32_t ulTriggerMode = 0;
	uint32_t ulHardwareTriggerEdge = 1;
	uint32_t ulHardwareTriggerCount = 0;
	uint32_t ulByTriggerNbrOfSeqToAcquire = 0;
	uint32_t ulDataFormat = 0;

	MFMARK("MSXE301x__AnalogInputInitAndStartSequenceEx");
	ret = modbus_call_MSXE301x__AnalogInputInitAndStartSequenceEx(modbus,
			ulNbrOfChannel,
			pulChannelList,
			pulGainArray,
			pulPolarityArray,
			ulConversionTime,
			ulConversionTimeUnit,
			ulNbrOfSequence,
			ulNbrMaxSequenceToTransfer,
			ulDelayMode,
			ulDelayTimeUnit,
			ulDelayValue,
			ulTriggerMask,
			ulTriggerMode,
			ulHardwareTriggerEdge,
			ulHardwareTriggerCount,
			ulByTriggerNbrOfSeqToAcquire,
			ulDataFormat,
			0,
			&CommandStatus
	);
	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSX301x__TransducerInitAndStartSequenceEx" );
}
//---------------------------------------------------------------------------------
int sample_MSXE301X__AnalogInputStopAndReleaseSequence(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	MFMARK("MSXE301x__AnalogInputStopAndReleaseSequence");
	ret = modbus_call_MSXE301x__AnalogInputStopAndReleaseSequence(modbus,Dummy,&CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE301x__AnalogInputStopAndReleaseSequence");
}
//---------------------------------------------------------------------------------
int sample_MSXE301X__AnalogInputStopAndReleaseSequenceEx(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	MFMARK("MSXE301x__AnalogInputStopAndReleaseSequenceEx");
	ret = modbus_call_MSXE301x__AnalogInputStopAndReleaseSequenceEx(modbus,Dummy,&CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE301x__AnalogInputStopAndReleaseSequenceEx");
}

//---------------------------------------------------------------------------------
static void usage(char* name)
{
	printf("%s ADDRESS tcp|udp\n",name);
}
//---------------------------------------------------------------------------------
void print_header(void)
{
        printf ("\n\n");
        printf ("\n+---------------------------------------------------+");
        printf ("\n| MSX-E301x MODBUS Sample                           |");
        printf ("\n+---------------------------------------------------+");
        printf ("\n| Press ESC to stop an acquisition                  |");
        printf ("\n+---------------------------------------------------+");
        printf ("\n\n");
}
//---------------------------------------------------------------------------------

int main(int argc, char** argv)
{
	struct modbus modbus;
	int contype = SOCK_DGRAM;

	if (argc != 3)
	{
		usage(argv[0]);
		exit(1);
	}

	if (0 == strcmp(argv[2],"tcp"))
	{
		contype=SOCK_STREAM;
	}
	else if(0 == strcmp(argv[2],"udp"))
	{
		contype=SOCK_DGRAM;
	}
	else
	{
		usage(argv[0]);
		exit(1);
	}
	print_header();
	modbus_init(&modbus);

	switch( modbus_connect(&modbus, argv[1], contype, 0) )
	{
	case 0: /* success */
		break;
	case -1: /* system error */
	addidata_network_perror("modbus_connect");
	exit(1);
	case -2: /* parameter is incorrect */
	printf("modbus_connect: incorrect parameter\n");
	exit(1);
	case -3: /* incorrect address */
	printf("incorrect address\n");
	exit(1);
	}

	assert( sample_MXCommon__GetModuleTypeEx(&modbus) == 0);
	assert( sample_MXCommon__GetTimeEx(&modbus) == 0);
	assert( sample_MXCommon__SetHardwareTriggerFilterTimeEx(&modbus, 1) == 0);
	assert( sample_MSXE301x__AnalogInputGetChannelTypeEx(&modbus) == 0);


	/* autorefresh */
	assert( sample_MSXE301X__TransducerInitAndStartAutoRefreshEx(&modbus) == 0);

	{
		key_thread_handle_t kt;
		unsigned int stop = 0;
		wait_key_input_thread_parameters_t ktparams;
		ktparams.character = 27; // ESC
		ktparams.flag = &stop;
		kt = addidata_start_key_thread(&ktparams);
		assert(kt!=NULL);
		{
			printf("Counter\tLow\tHigh");
			{
				int i;
				for(i=1;i<17;i++)
					printf("\t%d",i);
			}
			printf("\n");
			while(1)
			{
				assert( sample_MSXE301x__AnalogInputGetAutoRefreshValuesEx(&modbus) == 0);
				if(stop)
				{
					break;
				}
			}
		}
		addidata_stop_key_thread(kt);
	}
	assert( sample_MSXE301x__AnalogInputStopAndReleaseAutoRefreshEx(&modbus) == 0);

	/* sequence */
	printf("\nStarting now a sequence acquisition.\n");
	printf("Please use the monitor in the Web interface to watch the values.\n");
	printf ("Press ESC to stop the acquisition\n");
	assert( sample_MSXE301X__AnalogInputInitAndStartSequenceEx(&modbus) == 0 );
	fflush(stdout);
	addidata_console_wait_for_input();
	assert( sample_MSXE301X__AnalogInputStopAndReleaseSequenceEx(&modbus) == 0 );
	printf("\nSequence acquisition stopped.\n");

	assert( modbus_shutdown(&modbus) == 0);

	printf("\npress a key to end this sample\n");
	fflush(stdout);
	addidata_console_wait_for_input();
	return 0;
}
