/** @file Calibration.c
* Calibration : MX370x : Calibration sample
* 
*
* @author S. Weber
* @date 16.05.2006
* 
* @version  $LastChangedRevision$
* @version $LastChangedDate$
* 
* 
* @par LICENCE
* @verbatim
*  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
*        
*        ADDI-DATA GmbH 
*        Airport Boulevard B210
*        D-77836 Rheinmnster Germany 
*        Tel: +49(0)7229/1847-0 
*        Fax: +49(0)7229/1847-200
*        http://www.addi-data-com 
*        info@addi-data.com 
*        
*
* @endverbatim   
*
*
* @par UPDATES                                                                                                          
* @verbatim                                                                                                             
*  Date      : 01.06.06                                                                                                 
*  Author        : C. Guinot                                                                                            
*  Description   : Remove the "_" character in the SOAP parameter name
*                                                                                                                       
* @endverbatim     
*
*/ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>


#include "..\..\..\..\Interface_Library\MSXE301x.nsmap"
#include "..\UTIL\utility.h"
#include "..\UTIL\socket.h"



/** Test error numbers.
 *
 * Test error numbers of a soap function (soap_call_MSXE301x__CalibrationStart) and display the appropriated error message. 
 *
 *
 * @retval 0: No error.
 * @retval 1: Error.
 */
int i_CheckAndPrintCalibrationError (struct soap *soap, const char *soap_endpoint, unsigned long CalibrationMode, unsigned long Type, int soapCallError, struct MSXE301x__Response *ps_Response)
	{
	struct MXCommon__ByteArrayResponse s_ByteArrayResponse;

	if (!((soapCallError == 0) && (ps_Response->iReturnValue == 0)))
	   {
	   printf("\n soap_call_MSXE301x__CalibrationStart SOAP error: %d,Response error: %d\n", soapCallError,ps_Response->iReturnValue);
	   if (soapCallError != 0)
	      {
		  printf("\nSOAP Error : %s", *soap_faultstring(soap));
		  }
	   else
	      {
		  if ((ps_Response->iReturnValue == -1) || (ps_Response->iReturnValue <= -100) && 
		      (ps_Response->syserrno != 0))
	         {
			 soapCallError = soap_call_MXCommon__Strerror(soap, soap_endpoint, SOAP_ACTION, ps_Response->syserrno, &s_ByteArrayResponse);
			 if (soapCallError != 0)
			    {
			    printf("\nSOAP Error : %s", *soap_faultstring(soap));
				}
			 else
			    {
				printf("\nError : %s",s_ByteArrayResponse.sArray.__ptr);
				}
	         }
		  else
		     {
			 switch (ps_Response->iReturnValue)
			    {
				case -2:
				     switch (CalibrationMode & 0xFF)
					    {
						case 0: 
				             printf ("    Calibration mode 0 not available for mixing input types (voltage/current)\n");
							 break;

						case 1:
				             printf ("    Calibration mode 1 not available for current or mixing input types\n    (voltage/current)\n");
							 break;

						case 2:
				             printf ("    Channel group %d (channel %d to %d) not available.\n", (CalibrationMode >> 8), (CalibrationMode >> 8) * 4, ((CalibrationMode >> 8) * 4) + 3);
							 break;

						case 3:
						     if (Type == 0)
							    {
				                printf ("    Channel group %d (channel %d to %d) not available.\n", (CalibrationMode >> 8), (CalibrationMode >> 8) * 4, ((CalibrationMode >> 8) * 4) + 3);
								}
							 else
							    {
				                printf ("    Calibration via internal voltage source not available for the current\n    channel group %d (channel %d to %d).\n", (CalibrationMode >> 8), (CalibrationMode >> 8) * 4, ((CalibrationMode >> 8) * 4) + 3);
								}
							 break;
						}
					 break;

				case -3:
				     printf ("    Calibration gain selection error.\n");
					 break;

				case -4:
				     printf ("    Calibration value selection error.\n");
					 break;
				}
			 }
	      }

	   return 1;
	   }
	
	return 0;
	}


/*******************************************************************************************************/
/*                                       SOAP Functions                                                */               
/*******************************************************************************************************/



void v_Calibration (struct soap *ps_SoapEnv,PCHAR pc_SOAPIPAddress)
	{
	INT                                                  i_Error            = 0;
	ULONG                                               ul_Gain             = 0;
    ULONG                                               ul_LastStatus       = 0;
	ULONG                                               ul_CalibrationMode  = 0;
	ULONG                                               ul_Type             = 0;
    double                                               d_CalibrationValue [2][2] = {{9.9920, 4.992},{0.0,19.968}};
    struct MSXE301x__Response                            s_SingleResponse;
    struct MSXE301x__CalibrationGetCurrentStatusResponse s_Response;
	struct MSXE301x__AnalogInputGetChannelTypeResponse   s_GetChannelTypeResponse;
	CHAR * pc_Unit[2] = {"V","mA"};



	/******************************/
	/* Calibration mode selection */
	/******************************/

    do
       {
	   printf ("\nCalibration mode selection:\n\n");
	   printf (" 0: All channels calibration via a external calibration source (voltage/current)");
	   printf ("    Not available for mixing input types.\n");
	   printf (" 1: All channels calibration via the internal voltage calibration source.\n");
	   printf ("    Not available for current mixing input types.\n");
	   printf (" 2: Channel group 0 (channel 0 to 3) calibration via a external calibration\n");
	   printf ("    source (voltage/current).\n");
	   printf (" 3: Channel group 1 (channel 4 to 7) calibration via a external calibration\n");
	   printf ("    source (voltage/current).\n");
	   printf (" 4: Channel group 2 (channel 8 to 11) calibration via a external calibration\n");
	   printf ("    source (voltage/current).\n");
	   printf (" 5: Channel group 2 (channel 12 to 15) calibration via a external calibration\n");
	   printf ("    source (voltage/current).\n");
	   printf (" 6: Channel group 0 (channel 0 to 3) calibration via the internal voltage\n");
	   printf ("    calibration source. Not available for current inputs\n");
	   printf (" 7: Channel group 1 (channel 4 to 7) calibration via the internal voltage\n");
	   printf ("    calibration source. Not available for current inputs\n");
	   printf (" 8: Channel group 2 (channel 8 to 11) calibration via the internal voltage\n");
	   printf ("    calibration source. Not available for current inputs\n");
	   printf (" 9: Channel group 3 (channel 12 to 15) calibration via the internal voltage\n");
	   printf ("    calibration source. Not available for current inputs\n\n");
	   printf ("\nSelection : ");
       scanf("%lu",&ul_CalibrationMode);
	   }
	while((ul_CalibrationMode < 0) || (ul_CalibrationMode > 9));

	/*************************/
	/* Convert the selection */
	/*************************/

	switch (ul_CalibrationMode)
	   {
	   case 2:
	   case 3:
	   case 4:
	   case 5:
	        ul_CalibrationMode = ((ul_CalibrationMode - 2) << 8) | 0x2;
			break;

	   case 6:
	   case 7:
	   case 8:
	   case 9:
	        ul_CalibrationMode = ((ul_CalibrationMode - 6) << 8) | 0x3;
			break;
	   }


	/*************************/
    /* Get all channels type */
	/*************************/

	i_Error = soap_call_MSXE301x__AnalogInputGetChannelType(ps_SoapEnv,pc_SOAPIPAddress,SOAP_ACTION,0,&s_GetChannelTypeResponse);

	/***********************/
	/* Test if error occur */
	/***********************/

	if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_GetChannelTypeResponse.sResponse.iReturnValue, s_GetChannelTypeResponse.sResponse.syserrno, "soap_call_MSXE301x__AnalogInputGetChannelType", 0))
	   {
	   return;
	   }
	else
	   {
	   /***************************************************************/
	   /* Get the selected channel group input type (voltage/current) */
	   /***************************************************************/

	   ul_Type = s_GetChannelTypeResponse.ulType[(ul_CalibrationMode >> 8) * 4];
	   }


	/**********************************************/
	/* Calibration gain selection only if voltage */
	/**********************************************/

	if (ul_Type == 0)
	   {

       do
          {
	      printf("\nCalibration gain selection (1 or 2):");
          scanf("%lu",&ul_Gain);
	      }
	   while((ul_Gain != 1) && (ul_Gain != 2));
	   }
	else
		{
		printf("\nCalibration with gain 2");
		ul_Gain = 2;
		}


	/*************************/
	/* Start the calibration */
	/*************************/

    i_Error = soap_call_MSXE301x__CalibrationStart (ps_SoapEnv,
	                                                pc_SOAPIPAddress, 
	                                                SOAP_ACTION,
                                                    (unsigned long) ul_CalibrationMode,      // 0 : External soure. 1: Internal source
                                                    (unsigned long) ul_Gain,                 // Calibration gain selection
                                                    d_CalibrationValue[ul_Type][ul_Gain-1],  // Calibration value for the gain calibration
                                                    &s_SingleResponse);

	/***********************/
	/* Test if error occur */
	/***********************/

	if (i_CheckAndPrintCalibrationError (ps_SoapEnv, pc_SOAPIPAddress, ul_CalibrationMode, ul_Type, i_Error, &s_SingleResponse))
	   {
	   return;
	   }


	while (ul_LastStatus != 5)
       {
       Sleep (10);
           
       /**********************************/
       /* Get calibration current status */
       /**********************************/
           
	   i_Error = soap_call_MSXE301x__CalibrationGetCurrentStatus (ps_SoapEnv,
	                                                              pc_SOAPIPAddress,
                                                                  SOAP_ACTION,
                                                                  NULL,
                                                                  &s_Response);
           
	   /***********************/
	   /* Test if error occur */
	   /***********************/

	   if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_Response.sResponse.iReturnValue, s_Response.sResponse.syserrno, "soap_call_MSXE301x__CalibrationGetCurrentStatus", 1))
	      {
	      return;
	      }

	   /*********************************/
	   /* Test if current status change */
	   /*********************************/

	   if (s_Response.ulStatus != ul_LastStatus)
	      {
          /************************************/
          /* Test if start offset calibration */
          /************************************/

		  if ((s_Response.ulStatus & 0xFF) == 1)
		     {
		     printf ("\nSet calibration source on channel %d to 0%s and press after this any key\n",(s_Response.ulStatus >> 8) * 4, pc_Unit[ul_Type]);
		     }

          /**********************************/
          /* Test if start gain calibration */
          /**********************************/
          else if ((s_Response.ulStatus & 0xFF) == 3)
             {
             printf ("\nSet calibration source on channel %d to %9.5lf%s and press after this any key\n", (s_Response.ulStatus >> 8) * 4, d_CalibrationValue[ul_Type][ul_Gain-1],pc_Unit[ul_Type]);
             }

          /**********************************/
          /* Test if start gain calibration */
          /**********************************/
          else if ((s_Response.ulStatus & 0xFF) == 6)
             {
             printf ("\nSet calibration source on channel %d to %9.5lf%s and press after this any key\n", (s_Response.ulStatus >> 8) * 4, -d_CalibrationValue[ul_Type][ul_Gain-1],pc_Unit[ul_Type]);
             }

          /**************************************/
          /* Test if offset calibration started */
          /**************************************/
          else if ((s_Response.ulStatus & 0xFF) == 2)
             {
             printf ("\nChannel group %d (channel %d to %d) offset calibration started\n", s_Response.ulStatus >> 8, (s_Response.ulStatus >> 8) * 4, ((s_Response.ulStatus >> 8) * 4) + 3);
	         }

          /************************************/
          /* Test if gain calibration started */
          /************************************/
          else if ((s_Response.ulStatus & 0xFF) == 4)
             {
             printf ("\nChannel group %d (channel %d to %d) gain calibration started\n", s_Response.ulStatus >> 8, (s_Response.ulStatus >> 8) * 4, ((s_Response.ulStatus >> 8) * 4) + 3);
	         }

          /************************/
          /* Calibration conclude */
          /************************/
		  else
		     {
	         switch (s_Response.ulError)
	            {
	            case 0:
		             printf ("\n\nCalibration conclude");
				     break;

			    case 1:
		             printf ("\n\nUser calibration break occur");
			         break;

                case 2:
                case 3:
		             printf ("\n\nCan not calibrate the offset for channels 0 to 3");
			         break;


                case 4:
                case 5:
		             printf ("\n\nCan not calibrate the offset for channels 4 to 7");
			         break;

                case 6:
                case 7:
		             printf ("\n\nCan not calibrate the offset for channels 8 to 11");
			         break;

                case 8:
                case 9:
		             printf ("\n\nCan not calibrate the offset for channels 12 to 15");
			         break;


                case 10:
                case 11:
		             printf ("\n\nCan not calibrate the gain for channels 0 to 3");
			         break;


                case 12:
                case 13:
		             printf ("\n\nCan not calibrate the gain for channels 4 to 7");
			         break;

                case 14:
                case 15:
		             printf ("\n\nCan not calibrate the gain for channels 8 to 11");
				     break;

                case 16:
                case 17:
		             printf ("\n\nCan not calibrate the gain for channels 12 to 15");
				     break;
		        }
		     }
		  }
	   else
	      {
	      printf ("\rActualy calibration value = %9.5lf%s", s_Response.dCurrentValue,pc_Unit[ul_Type]);
	      }

	   ul_LastStatus = s_Response.ulStatus;


	   /*********************/
       /* Test if key press */
	   /*********************/
	   
	   if (kbhit ())
	      {
	      /******************/
	      /* Test if no ESC */
	      /******************/

	      if (getch () != 27)
	         {
             /***********************************/
             /* Test if calibration in progress */
             /***********************************/

             if (ul_LastStatus != 5)
                {
                /******************************/
                /* Test if next step selected */
                /******************************/

                if (((ul_LastStatus & 0xFF) == 1) || ((ul_LastStatus & 0xFF) == 3) || ((ul_LastStatus & 0xFF) == 6))
                   {
                   /******************************************/
                   /* Send the next calibration step command */
                   /******************************************/
                
	               soap_call_MSXE301x__CalibrationNextStep (ps_SoapEnv,
	                                                        pc_SOAPIPAddress, 
	                                                        SOAP_ACTION,
                                                            NULL,
                                                            &s_SingleResponse);
                 
                   }
                }
		     }
	      else
	         {
             /***********************************/
             /* Test if calibration in progress */
             /***********************************/

             if (ul_LastStatus != 5)
                {
                /***********************
                 * Stop the calibration 
                 ************************/

                soap_call_MSXE301x__CalibrationBreak (ps_SoapEnv,
	                                                  pc_SOAPIPAddress, 
	                                                  SOAP_ACTION,
                                                      NULL,
                                                      &s_SingleResponse);
                 
                }
             }
	      }
		/* 
		 * gSOAP doesn't release automatically the memory that 
		 * it allocates to deserialize data, that's why both 
		 * following functions have to be used to release the 
		 * allocated memory.
		 */
		soap_destroy(ps_SoapEnv);
		soap_end(ps_SoapEnv);		  
       }

	return;
    }


void	main(void)
{
	CHAR                                pc_IPAddressTemp[50];
	CHAR                                pc_IPAddress    [50];
	CHAR                                pc_SOAPIPAddress[50];
	BYTE                                 b_Exit                      = 0;
	INT                                  i_Error                     = 0;
	INT                                  i_ReturnValue               = 0;
	DWORD                               dw_Dummy                     = 0;
	DWORD                               dw_Cpt                       = 0;
	xsd__double                          d_Range                     = 0;
	xsd__unsignedLong                   dw_TransducerTypeSelection   = 0;
	struct soap                        *ps_SoapEnv                   = NULL;
	struct MXCommon__ByteArrayResponse   s_ByteArrayResponse;


	/**************************/
	/* Get the console handle */
	/**************************/
	ShowWindow(GetForegroundWindow (), SW_SHOWMAXIMIZED);

	/****************************/
	/* Print sample information */
	/****************************/
	printf("+---------------------------------------------------+\n");
	printf("| MSX-E301x Calibration                             |\n");
	printf("| MSX-E301x Calibration Test Program                |\n");
	printf("+---------------------------------------------------+\n");
	printf("| <ESC> exit sample                                 |\n");
	printf("+---------------------------------------------------+\n");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	   {
	   printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
	   gets(pc_IPAddressTemp);

	   v_UTILS_ClearNullInIPAddress(pc_IPAddressTemp,pc_IPAddress);
	   sprintf(pc_SOAPIPAddress,"%s:5555", pc_IPAddress);
	   printf("\nSOAP IP Address: %s\n", pc_SOAPIPAddress);

	   i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv,pc_SOAPIPAddress , SOAP_ACTION, &dw_Dummy,&s_ByteArrayResponse);
	   
	   if (!(i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_ByteArrayResponse.sResponse.iReturnValue, s_ByteArrayResponse.sResponse.syserrno, "soap_call_MXCommon__GetModuleType", 1)))
	      {
		  /*******************************/
		  /* Show the module inforamtion */
		  /*******************************/

	      printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);

		  /************************/
   		  /* Make the calibration */
		  /************************/

          v_Calibration (ps_SoapEnv,pc_SOAPIPAddress);
		  }

	   /* Close the SOAP connection */
	   i_SOAP_Close (ps_SoapEnv);
	   }

   
	/**********************************/
	/*Print End programme informations*/
	/**********************************/
	printf("\n\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");

	while (!(kbhit()));
}
